---- Localization

-- English (default)
local L = {
	["Bosses killed"] = "Bossed killed:",
	["Bosses alive"] = "Bosses alive:",
	["Will reset in"] = "Will reset in",
	["less than a minute"] = "less than a minute",
	["and"] = "and",
	["minutes"] = {
		["one"] = "minute",
		["many"] = "minutes"
	},
	["hours"] = {
		["one"] = "hour",
		["many"] = "hours"
	},
	["days"] = {
		["one"] = "day",
		["many"] = "days"
	}
}
local function Pluralize(number, localeTable)
	return number > 1 and localeTable.many or localeTable.one
end

----

local locale = GetLocale()

if locale == "frFR" then
elseif locale == "deDE" then
elseif locale == "esMX" then
elseif locale == "ruRU" then
	L = {
		["Bosses killed"] = "Убитые боссы:",
		["Bosses alive"] = "Живые боссы:",
		["Will reset in"] = "Сбросится через",
		["less than a minute"] = "несколько секунд",
		["and"] = "и",
		["minutes"] = {
			["one"] = "минуту",
			["few"] = "минуты",
			["many"] = "минут"
		},
		["hours"] = {
			["one"] = "час",
			["few"] = "часа",
			["many"] = "часов"
		},
		["days"] = {
			["one"] = "день",
			["few"] = "дня",
			["many"] = "дней"
		}
	}
	--- one -> n mod 10 is 1 and n mod 100 is not 11;
	--- few -> n mod 10 in 2..4 and n mod 100 not in 12..14;
	Pluralize = function(number, localeTable)
		if (number % 10 == 1) and (number % 100 ~= 11) then
			return number .. ' ' .. localeTable.one
		elseif (number % 10 >= 2 or number % 10 <= 4) and (number % 100 < 12 or number % 100 > 14) then
			return number .. ' ' .. localeTable.few
		else
			return number .. ' ' .. localeTable.many
		end
	end
elseif locale == "esES" then
elseif locale == "zhTW" then
elseif locale == "zhCN" then
elseif locale == "koKR" then
end


local eventFrame = CreateFrame("Frame", "AdventureGuideSavedInstances_EventFrame", UIParent)
eventFrame:Show()

local startTime = -1
local savedInstances = {}
local statusFrames = {}

local function UpdateSavedInstances() 
	savedInstances = {}
	for i = 1, GetNumSavedInstances() do
		local instanceName, instanceID, instanceReset, instanceDifficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, maxBosses, defeatedBosses = GetSavedInstanceInfo(i)
		if savedInstances[instanceName] == nil then
			savedInstances[instanceName] = {}
		end
		
		local difficulty = 'normal'
		
		if instanceDifficulty == 2 or instanceDifficulty == 5 or instanceDifficulty == 6 then
			difficulty = 'heroic'
		elseif instanceDifficulty == 23 then
			difficulty = 'mythic'
		end

		local bosses = {}
		for b = 1, maxBosses do
			local bossName, texture, isKilled, unknown4 = GetSavedInstanceEncounterInfo(i, b)
			table.insert(bosses, {
				name = bossName,
				isKilled = isKilled
			})
		end

		-- Violet Hold shows 8 bosses, but only 3 can be killed on single run
		if instanceID == 255628473 then
			maxBosses = 3
		end

		if locked then
			table.insert(savedInstances[instanceName], {
				bosses = bosses,
				instanceName = instanceName,
				instanceDifficulty = instanceDifficulty,
				difficulty = difficulty,
				difficultyName = difficultyName,
				maxBosses = maxBosses, 
				defeatedBosses = defeatedBosses,
				instanceReset = instanceReset,
				progress = defeatedBosses.."/"..maxBosses,
				complete = locked and defeatedBosses == maxBosses,
				locked = locked
			})
		end
	end
end

local function UpdateStatusFramePosition(instanceFrame)
	local savedFrames = statusFrames[instanceFrame:GetName()]
	local normalVisible = savedFrames and savedFrames['normal'] and savedFrames['normal']:IsShown()
	local heroicVisible = savedFrames and savedFrames['heroic'] and savedFrames['heroic']:IsShown()
	local mythicVisible = savedFrames and savedFrames['mythic'] and savedFrames['mythic']:IsShown()
	
	if mythicVisible then
		savedFrames['mythic']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", 4, -12)
	end
	
	if heroicVisible then
		if mythicVisible then
			savedFrames['heroic']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", -28, -12)
		else
			savedFrames['heroic']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", 4, -12)
		end
	end
	
	if normalVisible then
		if heroicVisible and mythicVisible then
			savedFrames['normal']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", -60, -23)		
		elseif heroicVisible or mythicVisible then
			savedFrames['normal']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", -28, -23)
		else		
			savedFrames['normal']:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", 4, -23)
		end
	end
end

local function GetTimeString(seconds)
	local result = ''

	local minute = 60
	local hour = minute * 60
	local day = hour * 24

	local days = 0
	local hours = 0
	local minutes = 0

	if seconds < minute then
		return L["less than a minute"]
	end

	if seconds > day then
		days = math.floor(seconds / day)
		seconds = seconds % day

	end

	if seconds > hour then
		hours = math.floor(seconds / hour)
		seconds = seconds % hour
	end

	if seconds > minute then
		minutes = math.floor(seconds / minute)
	end

	--- 

	if days > 0 then
		result = Pluralize(days, L['days'])
		-- result = days > 1 and days .. ' days' or days .. ' day'
	end

	if hours > 0 then
		if days > 0 and minutes > 0 then
			result = result .. ' '
		elseif days > 0 then
			result = result .. ' ' .. L['and'] .. ' '
		end
		result = result .. Pluralize(hours, L['hours'])
	end

	if minutes > 0 then
		if days > 0 or hours > 0 then
			result = result .. ' and '
		end	
		result = result .. Pluralize(minutes, L['minutes'])
	end

	return result
end

local function ShowTooltip(frame)
	local info = frame.instanceInfo

	GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:SetText(info.instanceName .. " (" .. info.difficultyName .. ")")

	if info.defeatedBosses > 0 then
		GameTooltip:AddLine(" ", 0, 0, 0)
		GameTooltip:AddLine(L["Bosses killed"], 1, 1, 1)
		for i, boss in ipairs(info.bosses) do
			if boss.isKilled then
				GameTooltip:AddLine("    " .. boss.name, 0, 1, 0)
			end
		end
	end
	
	if not info.complete then
		GameTooltip:AddLine(" ", 0, 0, 0)
		GameTooltip:AddLine(L["Bosses alive"], 1, 1, 1)
		for i, boss in ipairs(info.bosses) do
			if not boss.isKilled then
				GameTooltip:AddLine("    " .. boss.name, 1, 0, 0)
			end
		end		
	end
	GameTooltip:AddLine(" ", 0, 0, 0)
	GameTooltip:AddLine(L["Will reset in"] .. ' ' .. GetTimeString(info.instanceReset), 1, 1, 1)
	GameTooltip:Show()
end

local function hideTooltip(frame)
	GameTooltip:Hide()
end


local function CreateStatusFrame(instanceFrame, difficulty)
	local statusFrame = CreateFrame("Frame", nil, instanceFrame)
	statusFrame:Hide()

	statusFrame:SetScript("OnEnter", ShowTooltip)
	statusFrame:SetScript("OnLeave", hideTooltip)

	-- skull flag
	statusFrame.texture = statusFrame:CreateTexture(nil, "ARTWORK")
	statusFrame.texture:SetPoint("TOPLEFT")
	statusFrame:SetSize(38, 46)
	statusFrame.texture:SetTexture("Interface\\Minimap\\UI-DungeonDifficulty-Button")
	statusFrame.texture:SetSize(38, 46)

	statusFrame:SetPoint("BOTTOMRIGHT", instanceFrame, "BOTTOMRIGHT", 17, -12)
	
	if difficulty == 'mythic' then
		-- statusFrame.texture:SetTexCoord(0.25, 0.5, 0.0703125, 0.4296875) -- myth
		statusFrame.texture:SetTexCoord(0.30, 0.45, 0.0703125, 0.4296875) -- myth
	elseif difficulty == 'heroic' then
		statusFrame.texture:SetTexCoord(0.05, 0.20, 0.0703125, 0.4296875) -- heroic
	else
		statusFrame.texture:SetTexCoord(0.05, 0.20, 0.5703125, 0.9296875) -- normal
	end
	
	-- green check mark
	local completeFrame = CreateFrame("Frame", nil, statusFrame)
	completeFrame:Hide()
	
	completeFrame.texture = completeFrame:CreateTexture(nil, "ARTWORK", "GreenCheckMarkTemplate")
	completeFrame:SetSize(16, 16)
	
	if difficulty == 'normal' then
		completeFrame:SetPoint("BOTTOM", statusFrame, "BOTTOM", 0, 14)
	else
		completeFrame:SetPoint("BOTTOM", statusFrame, "BOTTOM", 0, 3)
	end
	
	completeFrame.texture:ClearAllPoints()
	completeFrame.texture:SetPoint("TOPLEFT")
	completeFrame.texture:Show()
			
	-- progress
	local progressFrame = statusFrame:CreateFontString(nil, nil, "GameFontNormalLeft")
	progressFrame:Hide()
	progressFrame:SetFont("Fonts\\ARIALN.TTF", 13)
	if difficulty == 'normal' then
		progressFrame:SetPoint("BOTTOM", statusFrame, "BOTTOM", 0, 19)
	else	
		progressFrame:SetPoint("BOTTOM", statusFrame, "BOTTOM", 0, 8)
	end
	progressFrame:SetTextColor(1, 1, 1)	
						
	statusFrame.completeFrame = completeFrame
	statusFrame.progressFrame = progressFrame
	
	if statusFrames[instanceFrame:GetName()] == nil then
		statusFrames[instanceFrame:GetName()] = {}
	end
	statusFrames[instanceFrame:GetName()][difficulty] = statusFrame
	
	return statusFrame
end

local function UpdateInstanceStatusFrame(instanceFrame)
	if statusFrames[instanceFrame:GetName()] then
		for difficulty, frame in pairs(statusFrames[instanceFrame:GetName()]) do
			frame:Hide()
		end	
	end

	local instances = savedInstances[instanceFrame.tooltipTitle]
	if instances == nil then
		return
	end

	for key, instance in ipairs(instances) do
		local frame = (statusFrames[instanceFrame:GetName()] and statusFrames[instanceFrame:GetName()][instance.difficulty]) or CreateStatusFrame(instanceFrame, instance.difficulty)
		if instance.complete then
			frame.completeFrame:Show()
			frame.progressFrame:Hide()
			frame:Show()
		elseif instance.progress then
			frame.completeFrame:Hide()
			frame.progressFrame:SetText(instance.progress)		
			frame.progressFrame:Show()			
			frame:Show()
		else
			frame:Hide()	
		end

		frame.instanceInfo = instance			
	end
	UpdateStatusFramePosition(instanceFrame)
end

-------------------

eventFrame:RegisterEvent("PLAYER_LOGIN")
local function OnEvent(self, event, ...)
	if event == "PLAYER_LOGIN" then
		startTime = GetTime()
	end
end
eventFrame:SetScript("OnEvent", OnEvent)

local function OnUpdate(self, elapsed)
	if startTime >= 0 and GetTime() - startTime > 2 and IsAddOnLoaded("Blizzard_EncounterJournal") then
		eventFrame:SetScript("OnUpdate", nil)
		startTime = nil

		-- UpdateSavedInstances()
		-- texplore(savedInstances)

		local function UpdateFrames()
			UpdateSavedInstances()
			local b1 = _G["EncounterJournalInstanceSelectScrollFrameScrollChildInstanceButton1"]
			if b1 then
				UpdateInstanceStatusFrame(b1)
			end
			for i = 1, 100 do
				local b = _G["EncounterJournalInstanceSelectScrollFrameinstance"..i]
				if b then
					UpdateInstanceStatusFrame(b)
				end
			end
		end
		hooksecurefunc("EncounterJournal_ListInstances", UpdateFrames)
		EncounterJournal_ListInstances()		
	end
end
eventFrame:SetScript("OnUpdate", OnUpdate)